package de.towerdefence.server.server;

import de.towerdefence.server.player.Player;
import de.towerdefence.server.session.Channel;
import de.towerdefence.server.session.SessionsService;
import lombok.AllArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;

@AllArgsConstructor
public abstract class JsonWebsocketHandler extends TextWebSocketHandler {
    private static final Logger logger = LoggerFactory.getLogger(JsonWebsocketHandler.class);
    public final Channel channel;
    protected final SessionsService sessionsService;
    protected final Map<WebSocketSession, Player> sessionPlayers = new ConcurrentHashMap<>();

    @Override
    public void afterConnectionEstablished(WebSocketSession session) {
        String jwt = session.getHandshakeHeaders().getFirst("Authorization");
        if (jwt == null){
            closeSession(session, CloseStatus.NOT_ACCEPTABLE);
            return;
        }
        Optional<Player> player = sessionsService.getSession(jwt, channel);
        if(player.isEmpty()){
            closeSession(session, CloseStatus.NOT_ACCEPTABLE);
            return;
        }
        sessionPlayers.put(session, player.get());
    }

    protected void closeSession(WebSocketSession session, CloseStatus reason){
        if(session.isOpen()){
            try{
                session.close(reason);
            } catch (Exception exception) {
                logger.info("Unable to Close the Session", exception);
            }
        }
    }

}
