package de.towerdefence.server.server;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.towerdefence.server.oas.ServerApi;
import de.towerdefence.server.oas.models.PlayerLoginData;
import de.towerdefence.server.oas.models.PlayerLoginSession;
import de.towerdefence.server.oas.models.PlayerRegistrationData;
import de.towerdefence.server.oas.models.ServerHealth;
import de.towerdefence.server.player.Player;
import de.towerdefence.server.player.PlayerRepository;
import de.towerdefence.server.player.PlayerService;
import de.towerdefence.server.session.Channel;
import de.towerdefence.server.session.SessionsService;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import java.security.NoSuchAlgorithmException;
import java.util.Optional;

@Controller
@RequestMapping("${openapi.api.base-path:/api/v1}")
public class ServerApiController implements ServerApi {

    @Autowired
    private PlayerRepository playerRepository;
    @Autowired
    private PlayerService playerService;
    @Autowired
    private SessionsService sessionsService;

    @Override
    public Optional<ObjectMapper> getObjectMapper() {
        return Optional.empty();
    }

    @Override
    public Optional<HttpServletRequest> getRequest() {
        return Optional.empty();
    }

    @Override
    public ResponseEntity<Void> playerRegister(PlayerRegistrationData body) {
        if(playerRepository.existsByUsername(body.getUsername())){
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }
        Player newPlayer = new Player();
        newPlayer.setUsername(body.getUsername());
        try{
            playerService.setPassword(newPlayer, body.getPassword());
        } catch (NoSuchAlgorithmException exception){
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
        playerRepository.save(newPlayer);
        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<PlayerLoginSession> playerLogin(PlayerLoginData body) {
        Player player = playerRepository.findByUsername(body.getUsername());
        if(player == null){
            return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
        }
        try {
            if(!playerService.checkPassword(player, body.getPassword())) {
                return new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
            }
        } catch (NoSuchAlgorithmException e) {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
        String token = sessionsService.createSession(player, Channel.CONNECTION);
        PlayerLoginSession session = new PlayerLoginSession();
        session.setUsername(player.getUsername());
        session.setToken(token);
        return new ResponseEntity<>(session, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<ServerHealth> serverGetHealthcheck() {
        ServerHealth health = new ServerHealth();
        health.setOkay(true);
        return ResponseEntity.ok(health);
    }
}
