package de.towerdefence.server.server.channels;

import de.towerdefence.server.match.confirmation.MatchConfirmationService;
import de.towerdefence.server.match.queue.MatchQueueService;
import de.towerdefence.server.server.JsonWebsocketHandler;
import de.towerdefence.server.server.channels.connection.ConnectionWebsocketHandler;
import de.towerdefence.server.server.channels.matchmaking.MatchmakingWebsocketHandler;
import de.towerdefence.server.server.channels.time.TimeWebsocketHandler;
import de.towerdefence.server.session.SessionsService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.config.annotation.EnableWebSocket;
import org.springframework.web.socket.config.annotation.WebSocketConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistry;

@AllArgsConstructor
@Configuration
@EnableWebSocket
public class WebsocketConfig implements WebSocketConfigurer {
    private static final String CHANNEL_BASE_PATH = "/ws/";
    @Autowired
    private final SessionsService sessionsService;
    @Autowired
    private  final MatchQueueService matchQueueService;
    @Autowired
    private  final MatchConfirmationService matchConfirmationService;

    @Override
    public void registerWebSocketHandlers(WebSocketHandlerRegistry registry) {
        registerJsonChannel(registry, new ConnectionWebsocketHandler(this.sessionsService));
        registerJsonChannel(registry, new MatchmakingWebsocketHandler(
                this.sessionsService,
                this.matchQueueService,
                this.matchConfirmationService
        ));
        registerJsonChannel(registry, new TimeWebsocketHandler(this.sessionsService));
    }

    private void registerJsonChannel(WebSocketHandlerRegistry registry, JsonWebsocketHandler handler){
        registry.addHandler(
                handler,
                CHANNEL_BASE_PATH + handler.channel.getJsonName()
        ).setAllowedOrigins("*");
    }
}
