package de.towerdefence.server.server.channels.connection;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.towerdefence.server.player.Player;
import de.towerdefence.server.server.JsonWebsocketHandler;
import de.towerdefence.server.server.channels.connection.in.RequestConnectionTokenMessage;
import de.towerdefence.server.server.channels.connection.out.ProvidedConnectionTokenMessage;
import de.towerdefence.server.session.Channel;
import de.towerdefence.server.session.SessionsService;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

public class ConnectionWebsocketHandler extends JsonWebsocketHandler {
    private final ObjectMapper objectMapper = new ObjectMapper();

    public ConnectionWebsocketHandler(SessionsService sessionsService) {
        super(Channel.CONNECTION, sessionsService);
    }

    @Override
    protected void handleTextMessage(WebSocketSession session, TextMessage message) {
        try{
            String payload = message.getPayload();
            switch ( objectMapper.readTree(payload).get("$id").asText()) {
                case RequestConnectionTokenMessage.MESSAGE_ID -> handleRequestConnectionToken(session, payload);
                default -> this.closeSession(session, CloseStatus.BAD_DATA);
            }
        } catch (Exception ignored) {
            this.closeSession(session, CloseStatus.BAD_DATA);
        }
    }

    private void handleRequestConnectionToken(WebSocketSession session, String payload) throws Exception {
        RequestConnectionTokenMessage msg = objectMapper.readValue(payload, RequestConnectionTokenMessage.class);
        Player player = this.sessionPlayers.get(session);
        String jwt = this.sessionsService.createSession(player, msg.getChannel());
        new ProvidedConnectionTokenMessage(msg.getChannel(), jwt).send(session);
    }
}
