package de.towerdefence.server.server.channels.time;

import de.towerdefence.server.server.JsonWebsocketHandler;
import de.towerdefence.server.session.Channel;
import de.towerdefence.server.session.SessionsService;
import org.springframework.web.socket.WebSocketSession;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.*;

public class TimeWebsocketHandler extends JsonWebsocketHandler {

    private final Map<WebSocketSession, ScheduledFuture<?>> sessionTaskMap = new ConcurrentHashMap<>();
    private static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);

    public TimeWebsocketHandler(SessionsService sessionsService) {
        super(Channel.TIME, sessionsService);
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) {
        super.afterConnectionEstablished(session);

        ScheduledFuture<?> scheduledTask = scheduler.scheduleAtFixedRate(
                () -> sendCurrentTime(session),
                0,
                1,
                TimeUnit.MILLISECONDS
        );
        sessionTaskMap.put(session, scheduledTask);
    }

    private void sendCurrentTime(WebSocketSession session) {
        ScheduledFuture<?> task = sessionTaskMap.get(session);
        try {
            if(!session.isOpen()){
                throw new RuntimeException("Session is not open");
            }
            new TimeMessage(System.currentTimeMillis()).send(session);
        } catch (RuntimeException | IOException e) {
            task.cancel(true);
            sessionTaskMap.remove(session);
        }
    }
}
