package de.towerdefence.server.server;

import de.towerdefence.server.IntegrationTest;
import de.towerdefence.server.oas.models.PlayerFilter;
import org.junit.jupiter.api.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.request.MockHttpServletRequestBuilder;

import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;

class GetAllPlayersTest extends IntegrationTest {

    private MockHttpServletRequestBuilder createGetAllPlayersRequest(String requestBody) {
        return MockMvcRequestBuilders.get(baseUri + "/admin/players")
                .contentType(MediaType.APPLICATION_JSON)
                .content(requestBody);
    }

    @Test
    void playersExist() throws Exception {
        PlayerFilter playerFilter = new PlayerFilter();
        playerFilter.setPage(0);
        playerFilter.setPageSize(10);
        playerFilter.setOrder(PlayerFilter.OrderEnum.DESCENDING);
        playerFilter.setUsername("");
        playerFilter.setSortBy("username");
        String requestBody = this.objectMapper.writeValueAsString(playerFilter);

        this.mvc.perform(createGetAllPlayersRequest(requestBody))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$").isArray())
                .andExpect(jsonPath("$[0]").exists());
    }

    @Test
    void playersSortedByAsc() throws Exception {

        PlayerFilter playerFilter = new PlayerFilter();
        playerFilter.setPage(0);
        playerFilter.setPageSize(10);
        playerFilter.setOrder(PlayerFilter.OrderEnum.ASCENDING);
        playerFilter.setUsername("");
        playerFilter.setSortBy("username");
        String requestBody = this.objectMapper.writeValueAsString(playerFilter);

        this.mvc.perform(createGetAllPlayersRequest(requestBody))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$[0].username").value("Alex"))
                .andExpect(jsonPath("$[1].username").value("Zorro"));
    }

    @Test
    void playersSortedByDesc() throws Exception {
        PlayerFilter playerFilter = new PlayerFilter();
        playerFilter.setPage(0);
        playerFilter.setPageSize(10);
        playerFilter.setOrder(PlayerFilter.OrderEnum.DESCENDING);
        playerFilter.setUsername("");
        playerFilter.setSortBy("username");
        String requestBody = this.objectMapper.writeValueAsString(playerFilter);

        this.mvc.perform(createGetAllPlayersRequest(requestBody))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$[1].username").value("Alex"))
                .andExpect(jsonPath("$[0].username").value("Zorro"));
    }

    @Test
    void playersFiltered() throws Exception {
        PlayerFilter playerFilter = new PlayerFilter();
        playerFilter.setPage(0);
        playerFilter.setPageSize(10);
        playerFilter.setOrder(PlayerFilter.OrderEnum.ASCENDING);
        playerFilter.setUsername("Alex");
        playerFilter.setSortBy("username");
        String requestBody = this.objectMapper.writeValueAsString(playerFilter);

        this.mvc.perform(createGetAllPlayersRequest(requestBody))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$[0].username").value("Alex"))
                .andExpect(jsonPath("$").isNotEmpty());
    }
}
